//
//  LocalProtocolHandler.m
//  test_webview
//
//  Created by Hans Knöchel on 10.02.18.
//  Copyright © 2018 Hans Knöchel. All rights reserved.
//

#import "LocalProtocolHandler.h"


@implementation LocalProtocolHandler

+ (BOOL)canInitWithRequest:(NSURLRequest *)request
{
  return [request.URL.scheme isEqualToString:@"file"];
}

+ (NSURLRequest *)canonicalRequestForRequest:(NSURLRequest *)request
{
  return request;
}

+ (BOOL)requestIsCacheEquivalent:(NSURLRequest *)a toRequest:(NSURLRequest *)b
{
  return NO;
}

- (void)startLoading
{
  id<NSURLProtocolClient> client = [self client];
  NSURLRequest *request = [self request];
  NSURL *url = [request URL];
  NSString *absolutePath = [url path];
  
  // Debug here: "po [NSURLProtocol propertyForKey:@"kContentInjection" inRequest:request];"
  // iOS 2.0-11.2: "myKey"
  // iOS 11.3: nil
  
  NSStringEncoding contentDataEncoding = [[[self class] propertyForKey:@"kContentDataEncoding" inRequest:request] unsignedIntegerValue];
  if (contentDataEncoding == 0) {
    contentDataEncoding = NSUTF8StringEncoding;
  }
  NSString *contentTextEncoding = [[self class] propertyForKey:@"kContentTextEncoding" inRequest:request];
  NSData *contentData = [[self class] propertyForKey:@"kContentData" inRequest:request];
  if (contentData == nil) {
    contentData = [NSData dataWithContentsOfFile:absolutePath];
    if (contentData == nil) {
      contentData = [NSData dataWithContentsOfFile:absolutePath];
      if (contentData == nil) {
        NSLog(@"[ERROR] Error loading %@", absolutePath);
        [client URLProtocol:self didFailWithError:[NSError errorWithDomain:NSURLErrorDomain code:NSURLErrorResourceUnavailable userInfo:nil]];
        [client URLProtocolDidFinishLoading:self];
        return;
      }
    }
  }
  NSString *contentMimeType = [[self class] propertyForKey:@"kContentMimeType" inRequest:request];
  if (contentMimeType == nil) {
    contentMimeType = @"text/html";
  }
  NSString *contentInjection = [[self class] propertyForKey:@"kContentInjection" inRequest:request];
  if ((contentInjection != nil) && [contentMimeType isEqualToString:@"text/html"]) {
    NSString *content = [[NSString alloc] initWithData:contentData encoding:contentDataEncoding];
    contentData = [content dataUsingEncoding:contentDataEncoding];
  }
  NSURLResponse *response = [[NSURLResponse alloc] initWithURL:url MIMEType:contentMimeType expectedContentLength:[contentData length] textEncodingName:contentTextEncoding];
  [client URLProtocol:self didReceiveResponse:response cacheStoragePolicy:NSURLCacheStorageNotAllowed];
  [client URLProtocol:self didLoadData:contentData];
  [client URLProtocolDidFinishLoading:self];
}

- (void)stopLoading
{
  // NO-OP
}

@end
